@@ -773,6 +773,34 @@ _ic_process_key_event  (BusInputContext       *context,
         }
     }
 
+    /* If I move the focus from the URL entry box of google-chrome
+     * to the text buffer of gnome-terminal,
+     * focus-in/focus-out of google-chrome is caused after
+     * focus-in of gonme-terminal and gnome-terminal loses the focus.
+     * The following focus events are received in ibusimcontext:
+     * 1) (gnome-terminal:445): IBUS-WARNING **: 15:32:36:717  focus_in
+     * 2) (google-chrome:495): IBUS-WARNING **: 15:32:36:866  focus_out
+     * 3) (google-chrome:495): IBUS-WARNING **: 15:32:36:875  focus_in
+     * 4) (google-chrome:495): IBUS-WARNING **: 15:32:36:890  focus_out
+     * In 2), Just return because focused_context is not google-chrome.
+     * In 3), focused_context is changed from gnome-terminal to google-chrome
+     * In 4), focused_context is changed from google-chrome to faked_context.
+     *
+     * It seems google-chrome has a popup window of the prediction of URL
+     * and async focus-in/focus-out.
+     */
+    if (context->has_focus && context->engine == NULL &&
+        context->fake == FALSE) {
+        BusInputContext *focused_context =
+                bus_ibus_impl_get_focused_input_context (BUS_DEFAULT_IBUS);
+
+        if (focused_context != NULL && context != focused_context &&
+            (context->capabilities & IBUS_CAP_FOCUS) != 0) {
+            context->has_focus = FALSE;
+            bus_input_context_focus_in (context);
+        }
+    }
+
     /* ignore key events, if it is a fake input context */
     if (context->has_focus && context->engine && context->fake == FALSE) {
         bus_engine_proxy_process_key_event (context->engine,
@@ -1123,6 +1151,47 @@ bus_input_context_service_method_call (IBusService            *service,
     g_return_if_reached ();
 }
 
+static void
+_ic_set_content_type (BusInputContext *context,
+                      GVariant        *value)
+{
+    guint purpose = 0;
+    guint hints = 0;
+
+    g_variant_get (value, "(uu)", &purpose, &hints);
+    if (purpose != context->purpose || hints != context->hints) {
+        GError *error;
+        gboolean retval;
+
+        context->purpose = purpose;
+        context->hints = hints;
+
+        if (context->has_focus && context->engine)
+            bus_engine_proxy_set_content_type (context->engine,
+                                               purpose,
+                                               hints);
+
+        if (context->has_focus) {
+            g_signal_emit (context,
+                           context_signals[SET_CONTENT_TYPE],
+                           0,
+                           context->purpose,
+                           context->hints);
+        }
+
+        error = NULL;
+        retval = bus_input_context_property_changed (context,
+                                                     "ContentType",
+                                                     value,
+                                                     &error);
+        if (!retval) {
+            g_warning ("Failed to emit PropertiesChanged signal: %s",
+                       error->message);
+            g_error_free (error);
+        }
+    }
+}
+
 static gboolean
 bus_input_context_service_set_property (IBusService     *service,
                                         GDBusConnection *connection,
@@ -1147,41 +1216,7 @@ bus_input_context_service_set_property (IBusService     *service,
 
     if (g_strcmp0 (property_name, "ContentType") == 0) {
         BusInputContext *context = (BusInputContext *) service;
-        guint purpose = 0;
-        guint hints = 0;
-
-        g_variant_get (value, "(uu)", &purpose, &hints);
-        if (purpose != context->purpose || hints != context->hints) {
-            GError *error;
-            gboolean retval;
-
-            context->purpose = purpose;
-            context->hints = hints;
-
-            if (context->has_focus && context->engine)
-                bus_engine_proxy_set_content_type (context->engine,
-                                                   purpose,
-                                                   hints);
-
-            if (context->has_focus) {
-                g_signal_emit (context,
-                               context_signals[SET_CONTENT_TYPE],
-                               0,
-                               context->purpose,
-                               context->hints);
-            }
-
-            error = NULL;
-            retval = bus_input_context_property_changed (context,
-                                                         "ContentType",
-                                                         value,
-                                                         &error);
-            if (!retval) {
-                g_warning ("Failed to emit PropertiesChanged signal: %s",
-                           error->message);
-                g_error_free (error);
-            }
-        }
+        _ic_set_content_type (context, value);
         return TRUE;
     }
 
@@ -2493,3 +2528,17 @@ bus_input_context_get_content_type (BusInputContext *context,
     *purpose = context->purpose;
     *hints = context->hints;
 }
+
+void
+bus_input_context_set_content_type (BusInputContext *context,
+                                    guint            purpose,
+                                    guint            hints)
+{
+    GVariant *value;
+
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+
+    value = g_variant_ref_sink (g_variant_new ("(uu)", purpose, hints));
+    _ic_set_content_type (context, value);
+    g_variant_unref (value);
+}
