@@ -1,8 +1,8 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2013 Red Hat, Inc.
+ * Copyright (C) 2008-2014 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2014 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -398,6 +398,20 @@ bus_panel_proxy_update_property (BusPanelProxy  *panel,
                        -1, NULL, NULL, NULL);
 }
 
+void
+bus_panel_proxy_set_content_type (BusPanelProxy  *panel,
+                                  guint           purpose,
+                                  guint           hints)
+{
+    g_assert (BUS_IS_PANEL_PROXY (panel));
+
+    g_dbus_proxy_call ((GDBusProxy *)panel,
+                       "ContentType",
+                       g_variant_new ("(uu)", purpose, hints),
+                       G_DBUS_CALL_FLAGS_NONE,
+                       -1, NULL, NULL, NULL);
+}
+
 #define DEFINE_FUNC(name)                                       \
     static void                                                 \
     bus_panel_proxy_##name (BusPanelProxy *panel)               \
@@ -575,6 +589,20 @@ _context_destroy_cb (BusInputContext *context,
     bus_panel_proxy_focus_out (panel, context);
 }
 
+static void
+_context_set_content_type_cb (BusInputContext *context,
+                              guint            purpose,
+                              guint            hints,
+                              BusPanelProxy   *panel)
+{
+    g_assert (BUS_IS_INPUT_CONTEXT (context));
+    g_assert (BUS_IS_PANEL_PROXY (panel));
+
+    g_return_if_fail (panel->focused_context == context);
+
+    bus_panel_proxy_set_content_type (panel, purpose, hints);
+}
+
 #define DEFINE_FUNCTION(name)                                   \
     static void _context_##name##_cb (BusInputContext *context, \
                                       BusPanelProxy   *panel)   \
@@ -629,6 +657,8 @@ static const struct {
     { "engine-changed",             G_CALLBACK (_context_state_changed_cb) },
 
     { "destroy",                    G_CALLBACK (_context_destroy_cb) },
+
+    { "set-content-type",           G_CALLBACK (_context_set_content_type_cb) },
 };
 
 void
@@ -663,6 +693,10 @@ bus_panel_proxy_focus_in (BusPanelProxy     *panel,
                           input_context_signals[i].callback,
                           panel);
     }
+
+    guint purpose, hints;
+    bus_input_context_get_content_type (context, &purpose, &hints);
+    bus_panel_proxy_set_content_type (panel, purpose, hints);
 }
 
 void
