@@ -1,23 +1,23 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus - The Input Bus
- * Copyright (C) 2008-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2008-2010 Red Hat, Inc.
+ * Copyright (C) 2008-2013 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2008-2013 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
+ * USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -95,6 +95,8 @@ static GtkWidget *_input_widget = NULL;
 static void     ibus_im_context_class_init  (IBusIMContextClass    *class);
 static void     ibus_im_context_class_fini  (IBusIMContextClass    *class);
 static void     ibus_im_context_init        (GObject               *obj);
+static void     ibus_im_context_notify      (GObject               *obj,
+                                             GParamSpec            *pspec);
 static void     ibus_im_context_finalize    (GObject               *obj);
 static void     ibus_im_context_reset       (GtkIMContext          *context);
 static gboolean ibus_im_context_filter_keypress
@@ -150,6 +152,7 @@ static gboolean _slave_delete_surrounding_cb
                                              IBusIMContext      *context);
 static void     _request_surrounding_text   (IBusIMContext      *context);
 static void     _create_fake_input_context  (void);
+static void     _set_content_type           (IBusIMContext      *context);
 
 
 
@@ -330,6 +333,26 @@ _request_surrounding_text (IBusIMContext *context)
     }
 }
 
+static void
+_set_content_type (IBusIMContext *context)
+{
+#if GTK_CHECK_VERSION (3, 6, 0)
+    if (context->ibuscontext != NULL) {
+        GtkInputPurpose purpose;
+        GtkInputHints hints;
+
+        g_object_get (G_OBJECT (context),
+                      "input-purpose", &purpose,
+                      "input-hints", &hints,
+                      NULL);
+
+        ibus_input_context_set_content_type (context->ibuscontext,
+                                             purpose,
+                                             hints);
+    }
+#endif
+}
+
 
 static gint
 _key_snooper_cb (GtkWidget   *widget,
@@ -469,7 +492,13 @@ daemon_name_appeared (GDBusConnection *connection,
                       const gchar     *owner,
                       gpointer         data)
 {
-    _daemon_is_running = TRUE;
+    /* If ibus-daemon is running and run ssh -X localhost,
+     * daemon_name_appeared() is called but ibus_get_address() == NULL
+     * because the hostname and display number are different between
+     * ibus-daemon and clients. So IBusBus would not be connected and
+     * ibusimcontext->ibuscontext == NULL and ibusimcontext->events_queue
+     * could go beyond MAX_QUEUED_EVENTS . */
+    _daemon_is_running = (ibus_get_address () != NULL);
 }
 
 static void
@@ -499,6 +528,7 @@ ibus_im_context_class_init (IBusIMContextClass *class)
     im_context_class->set_cursor_location = ibus_im_context_set_cursor_location;
     im_context_class->set_use_preedit = ibus_im_context_set_use_preedit;
     im_context_class->set_surrounding = ibus_im_context_set_surrounding;
+    gobject_class->notify = ibus_im_context_notify;
     gobject_class->finalize = ibus_im_context_finalize;
 
     _signal_commit_id =
@@ -549,7 +579,11 @@ ibus_im_context_class_init (IBusIMContextClass *class)
 
     /* init bus object */
     if (_bus == NULL) {
-        ibus_set_display (gdk_display_get_name (gdk_display_get_default ()));
+        const gchar *dname = gdk_display_get_name (gdk_display_get_default ());
+        /* ibus-daemon uses DISPLAY variable. */
+        if (g_strcmp0 (dname, "Wayland") == 0)
+            dname = g_getenv ("DISPLAY");
+        ibus_set_display (dname);
         _bus = ibus_bus_new_async ();
 
         /* init the global fake context */
@@ -691,6 +725,18 @@ ibus_im_context_init (GObject *obj)
     g_signal_connect (_bus, "connected", G_CALLBACK (_bus_connected_cb), obj);
 }
 
+static void
+ibus_im_context_notify (GObject    *obj,
+                        GParamSpec *pspec)
+{
+    IDEBUG ("%s", __FUNCTION__);
+
+    if (g_strcmp0 (pspec->name, "input-purpose") == 0 ||
+        g_strcmp0 (pspec->name, "input-hints") == 0) {
+        _set_content_type (IBUS_IM_CONTEXT (obj));
+    }
+}
+
 static void
 ibus_im_context_finalize (GObject *obj)
 {
@@ -815,12 +861,15 @@ ibus_im_context_focus_in (GtkIMContext *context)
         }
     }
 
-    if (_focus_im_context != NULL) {
-        g_assert (_focus_im_context != context);
-        gtk_im_context_focus_out (_focus_im_context);
-        g_assert (_focus_im_context == NULL);
-    }
-    else {
+    /* Do not call gtk_im_context_focus_out() here.
+     * google-chrome's notification popup window (Pushbullet)
+     * takes the focus and the popup window disappears.
+     * So other applications lose the focus because
+     * ibusimcontext->has_focus is FALSE if
+     * gtk_im_context_focus_out() is called here when
+     * _focus_im_context != context.
+     */
+    if (_focus_im_context == NULL) {
         /* focus out fake context */
         if (_fake_context != NULL) {
             ibus_input_context_focus_out (_fake_context);
@@ -829,6 +878,7 @@ ibus_im_context_focus_in (GtkIMContext *context)
 
     ibusimcontext->has_focus = TRUE;
     if (ibusimcontext->ibuscontext) {
+        _set_content_type (ibusimcontext);
         ibus_input_context_focus_in (ibusimcontext->ibuscontext);
     }
 
@@ -860,7 +910,6 @@ ibus_im_context_focus_out (GtkIMContext *context)
         return;
     }
 
-    g_assert (context == _focus_im_context);
     g_object_remove_weak_pointer ((GObject *) context,
                                   (gpointer *) &_focus_im_context);
     _focus_im_context = NULL;
@@ -1529,6 +1578,13 @@ _create_input_context_done (IBusBus       *bus,
         ibus_input_context_set_capabilities (ibusimcontext->ibuscontext, ibusimcontext->caps);
 
         if (ibusimcontext->has_focus) {
+            /* The time order is _create_input_context() ->
+             * ibus_im_context_notify() -> ibus_im_context_focus_in() ->
+             * _create_input_context_done()
+             * so _set_content_type() is called at the beginning here
+             * because ibusimcontext->ibuscontext == NULL before. */
+            _set_content_type (ibusimcontext);
+
             ibus_input_context_focus_in (ibusimcontext->ibuscontext);
             _set_cursor_location_internal (ibusimcontext);
         }
