@@ -1,8 +1,8 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 /* vim:set et sts=4: */
 /* ibus
- * Copyright (C) 2007-2010 Peng Huang <shawn.p.huang@gmail.com>
- * Copyright (C) 2007-2010 Red Hat, Inc.
+ * Copyright (C) 2007-2015 Peng Huang <shawn.p.huang@gmail.com>
+ * Copyright (C) 2007-2015 Red Hat, Inc.
  *
  * main.c:
  *
@@ -116,7 +116,7 @@ static gint     g_debug_level = 0;
 
 static IBusBus *_bus = NULL;
 
-static gboolean _use_sync_mode = FALSE;
+static gboolean _use_sync_mode = TRUE;
 
 static void
 _xim_preedit_start (XIMS xims, const X11IC *x11ic)
@@ -1015,7 +1015,8 @@ _init_ibus (void)
     g_signal_connect (_bus, "disconnected",
                         G_CALLBACK (_bus_disconnected_cb), NULL);
 
-    _use_sync_mode = _get_boolean_env ("IBUS_ENABLE_SYNC_MODE", FALSE);
+    /* https://github.com/ibus/ibus/issues/1713 */
+    _use_sync_mode = _get_boolean_env ("IBUS_ENABLE_SYNC_MODE", TRUE);
 }
 
 static void
@@ -1069,9 +1070,13 @@ _xim_init_IMdkit ()
         sizeof (ims_encodings)/sizeof (XIMEncoding) - 1;
     encodings.supported_encodings = ims_encodings;
 
-    _xims = IMOpenIM(GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
+    _xims = IMOpenIM (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
         IMModifiers, "Xi18n",
-        IMServerWindow, GDK_WINDOW_XWINDOW(win),
+#if GTK_CHECK_VERSION (3, 0, 0)
+        IMServerWindow, GDK_WINDOW_XID (win),
+#else
+        IMServerWindow, GDK_WINDOW_XWINDOW (win),
+#endif
         IMServerName, _server_name != NULL ? _server_name : "ibus",
         IMLocale, _locale != NULL ? _locale : LOCALES_STRING,
         IMServerTransport, "X/",
@@ -1133,6 +1138,13 @@ main (int argc, char **argv)
     gint option_index = 0;
     gint c;
 
+    /* GDK_DISPLAY_XDISPLAY() and GDK_WINDOW_XID() does not work
+     * with GdkWaylandDisplay.
+     */
+#if GTK_CHECK_VERSION (3, 10, 0)
+    gdk_set_allowed_backends ("x11");
+#endif
+
     gtk_init (&argc, &argv);
     XSetErrorHandler (_xerror_handler);
 
