@@ -20,6 +20,7 @@ print_engines (const GList *engines)
     }
 }
 
+#ifndef IBUS_DISABLE_DEPRECATED
 static void
 test_list_active_engines (void)
 {
@@ -32,6 +33,7 @@ test_list_active_engines (void)
     g_list_foreach (engines, (GFunc) g_object_unref, NULL);
     g_list_free (engines);
 }
+#endif /* IBUS_DISABLE_DEPRECATED */
 
 static void
 test_list_engines (void)
@@ -46,6 +48,16 @@ test_list_engines (void)
     g_list_free (engines);
 }
 
+static void
+name_owner_changed_cb (IBusBus  *bus,
+                       gchar    *name,
+                       gchar    *old,
+                       gchar    *new,
+                       gpointer  data)
+{
+    g_debug ("%s: bus=%s, old=%s, new=%s", G_STRFUNC, name, old, new);
+}
+
 static void call_next_async_function (void);
 
 static void
@@ -54,10 +66,34 @@ finish_request_name_async (GObject *source_object,
                            gpointer user_data)
 {
     GError *error = NULL;
-    guint id = ibus_bus_request_name_async_finish (bus,
-                                                   res,
-                                                   &error);
+    guint id = ibus_bus_request_name_async_finish (bus, res, &error);
+
     g_assert (id != 0);
+
+    g_debug ("request name returned %d: ", id);
+
+    switch (id) {
+    case IBUS_BUS_REQUEST_NAME_REPLY_PRIMARY_OWNER:
+        g_debug ("got ownership");
+        break;
+    case IBUS_BUS_REQUEST_NAME_REPLY_IN_QUEUE:
+        g_debug ("got queued");
+        break;
+    case IBUS_BUS_REQUEST_NAME_REPLY_EXISTS:
+        g_debug ("request already in queue");
+        break;
+    case IBUS_BUS_REQUEST_NAME_REPLY_ALREADY_OWNER:
+        g_debug ("already owner");
+        break;
+    default:
+        g_assert_not_reached ();
+    }
+
+    if (error) {
+        g_warning ("Error %s: %s", G_STRFUNC, error->message);
+        g_error_free (error);
+    }
+
     g_debug ("ibus_bus_request_name_async_finish: OK");
     call_next_async_function ();
 }
@@ -65,9 +101,15 @@ finish_request_name_async (GObject *source_object,
 static void
 start_request_name_async (void)
 {
+    ibus_bus_set_watch_dbus_signal (bus, TRUE);
+    ibus_bus_set_watch_ibus_signal (bus, TRUE);
+
+    g_signal_connect (bus, "name-owner-changed",
+                      (GCallback) name_owner_changed_cb, NULL);
+
     ibus_bus_request_name_async (bus,
                                  "org.freedesktop.IBus.IBusBusTest",
-                                 0,
+                                 IBUS_BUS_NAME_FLAG_REPLACE_EXISTING,
                                  -1, /* timeout */
                                  NULL, /* cancellable */
                                  finish_request_name_async,
@@ -353,6 +395,7 @@ start_list_engines_async (void)
                                  NULL); /* user_data */
 }
 
+#ifndef IBUS_DISABLE_DEPRECATED
 static void
 finish_list_active_engines_async (GObject *source_object,
                                   GAsyncResult *res,
@@ -447,6 +490,7 @@ start_is_global_engine_enabled_async (void)
                                              finish_is_global_engine_enabled_async,
                                              NULL); /* user_data */
 }
+#endif /* IBUS_DISABLE_DEPRECATED */
 
 static void
 finish_get_global_engine_async (GObject *source_object,
@@ -522,6 +566,242 @@ start_preload_engines_async (void)
             NULL); /* user_data */
 }
 
+static void
+test_get_address (void)
+{
+    GVariant *result;
+
+    result = ibus_bus_get_ibus_property (bus, "Address");
+    g_variant_get_string (result, NULL);
+    g_variant_unref (result);
+}
+
+static void
+test_get_current_input_context (void)
+{
+    GVariant *result;
+
+    result = ibus_bus_get_ibus_property (bus, "CurrentInputContext");
+    g_variant_get_string (result, NULL);
+    g_variant_unref (result);
+}
+
+static void
+test_get_engines (void)
+{
+    GVariant *result, *var;
+    GVariantIter *iter;
+    GList *engines = NULL;
+
+    result = ibus_bus_get_ibus_property (bus, "Engines");
+    iter = g_variant_iter_new (result);
+    while (g_variant_iter_loop (iter, "v", &var)) {
+        IBusSerializable *serializable = ibus_serializable_deserialize (var);
+        g_object_ref_sink (serializable);
+        engines = g_list_append (engines, serializable);
+    }
+    g_variant_iter_free (iter);
+    g_variant_unref (result);
+
+    print_engines (engines);
+
+    g_list_foreach (engines, (GFunc) g_object_unref, NULL);
+    g_list_free (engines);
+}
+
+static void
+test_get_global_engine (void)
+{
+    GVariant *result, *obj;
+    IBusEngineDesc *desc = NULL;
+
+    if (!ibus_bus_set_global_engine (bus, "xkb:us::eng"))
+        return;
+
+    result = ibus_bus_get_ibus_property (bus, "GlobalEngine");
+    g_assert (result);
+
+    obj = g_variant_get_variant (result);
+    g_assert (obj);
+
+    desc = IBUS_ENGINE_DESC (ibus_serializable_deserialize (obj));
+    g_assert (desc);
+    g_assert_cmpstr (ibus_engine_desc_get_name (desc), ==, "xkb:us::eng");
+
+    g_variant_unref (obj);
+    g_variant_unref (result);
+
+    g_object_unref (desc);
+}
+
+static void
+test_set_preload_engines (void)
+{
+    const gchar *preload_engines[] = { "xkb:us::eng", "xkb:jp::jpn", NULL };
+    GVariant *variant;
+
+    variant = g_variant_new_strv (preload_engines, -1);
+    ibus_bus_set_ibus_property (bus, "PreloadEngines", variant);
+}
+
+static void
+finish_get_address_async (GObject      *source_object,
+                          GAsyncResult *res,
+                          gpointer      user_data)
+{
+    GError *error = NULL;
+    GVariant *result;
+
+    result = ibus_bus_get_ibus_property_async_finish (bus, res, &error);
+    g_variant_get_string (result, NULL);
+    g_variant_unref (result);
+    g_debug ("finish_get_address_async: OK");
+    call_next_async_function ();
+}
+
+static void
+start_get_address_async (void)
+{
+    ibus_bus_get_ibus_property_async (
+            bus,
+            "Address",
+            -1, /* timeout */
+            NULL, /* cancellable */
+            finish_get_address_async,
+            NULL); /* user_data */
+}
+
+static void
+finish_get_current_input_context_async (GObject      *source_object,
+                                        GAsyncResult *res,
+                                        gpointer      user_data)
+{
+    GError *error = NULL;
+    GVariant *result;
+
+    result = ibus_bus_get_ibus_property_async_finish (bus, res, &error);
+    g_variant_get_string (result, NULL);
+    g_variant_unref (result);
+    g_debug ("finish_get_current_input_context_async: OK");
+    call_next_async_function ();
+}
+
+static void
+start_get_current_input_context_async (void)
+{
+    ibus_bus_get_ibus_property_async (
+            bus,
+            "CurrentInputContext",
+            -1, /* timeout */
+            NULL, /* cancellable */
+            finish_get_current_input_context_async,
+            NULL); /* user_data */
+}
+
+static void
+finish_get_engines_async (GObject      *source_object,
+                          GAsyncResult *res,
+                          gpointer      user_data)
+{
+    GError *error = NULL;
+    GVariant *result, *var;
+    GVariantIter *iter;
+    GList *engines = NULL;
+
+    result = ibus_bus_get_ibus_property_async_finish (bus, res, &error);
+    iter = g_variant_iter_new (result);
+    while (g_variant_iter_loop (iter, "v", &var)) {
+        IBusSerializable *serializable = ibus_serializable_deserialize (var);
+        g_object_ref_sink (serializable);
+        engines = g_list_append (engines, serializable);
+    }
+    g_variant_iter_free (iter);
+    g_variant_unref (result);
+
+    print_engines (engines);
+
+    g_list_foreach (engines, (GFunc) g_object_unref, NULL);
+    g_list_free (engines);
+
+    g_debug ("finish_get_engines_async: OK");
+    call_next_async_function ();
+}
+
+static void
+start_get_engines_async (void)
+{
+    ibus_bus_get_ibus_property_async (
+            bus,
+            "Engines",
+            -1, /* timeout */
+            NULL, /* cancellable */
+            finish_get_engines_async,
+            NULL); /* user_data */
+}
+
+static void
+finish_get_prop_global_engine_async (GObject *source_object,
+                                     GAsyncResult *res,
+                                     gpointer user_data)
+{
+    GError *error = NULL;
+    GVariant *result, *obj;
+    IBusEngineDesc *desc = NULL;
+
+    result = ibus_bus_get_ibus_property_async_finish (bus, res, &error);
+    obj = g_variant_get_variant (result);
+    desc = IBUS_ENGINE_DESC (ibus_serializable_deserialize (obj));
+    g_variant_unref (obj);
+    g_variant_unref (result);
+
+    if (desc)
+        g_object_unref (desc);
+
+    g_debug ("finish_get_prop_global_engine_async: OK");
+    call_next_async_function ();
+}
+
+static void
+start_get_prop_global_engine_async (void)
+{
+    ibus_bus_get_ibus_property_async (
+            bus,
+            "GlobalEngine",
+            -1, /* timeout */
+            NULL, /* cancellable */
+            finish_get_prop_global_engine_async,
+            NULL); /* user_data */
+}
+
+static void
+finish_set_preload_engines_async (GObject      *source_object,
+                                  GAsyncResult *res,
+                                  gpointer      user_data)
+{
+    GError *error = NULL;
+
+    ibus_bus_set_ibus_property_async_finish (bus, res, &error);
+    g_debug ("finish_set_preload_engines_async: OK");
+    call_next_async_function ();
+}
+
+static void
+start_set_preload_engines_async (void)
+{
+    const gchar *preload_engines[] = { "xkb:us::eng", "xkb:jp::jpn", NULL };
+    GVariant *variant;
+
+    variant = g_variant_new_strv (preload_engines, -1);
+    ibus_bus_set_ibus_property_async (
+            bus,
+            "PreloadEngines",
+            variant,
+            -1, /* timeout */
+            NULL, /* cancellable */
+            finish_set_preload_engines_async,
+            NULL); /* user_data */
+}
+
 static void
 finish_exit_async (GObject *source_object,
                    GAsyncResult *res,
@@ -579,8 +859,6 @@ test_get_engines_by_names (void)
         g_assert_cmpstr (names[i], ==, ibus_engine_desc_get_name (*p));
         i++;
         g_object_unref (*p);
-        // The ref should be zero, *p is released.
-        g_assert (!IBUS_IS_ENGINE_DESC (*p));
     }
 
     // The last engine does not exist.
@@ -619,6 +897,11 @@ call_next_async_function (void)
         start_set_global_engine_async,
         start_get_global_engine_async,
         start_preload_engines_async,
+        start_get_address_async,
+        start_get_current_input_context_async,
+        start_get_engines_async,
+        start_get_prop_global_engine_async,
+        start_set_preload_engines_async,
         start_exit_async,
     };
     static guint index = 0;
@@ -652,18 +935,29 @@ main (gint    argc,
       gchar **argv)
 {
     gint result;
-    g_type_init ();
-    g_test_init (&argc, &argv, NULL);
     ibus_init ();
+    g_test_init (&argc, &argv, NULL);
     bus = ibus_bus_new ();
     g_object_unref (bus);
     bus = ibus_bus_new (); // crosbug.com/17293
 
+    if (!ibus_bus_is_connected (bus)) {
+        g_warning ("Not connected to ibus-daemon");
+        g_object_unref (bus);
+        return -1;
+    }
+
     g_test_add_func ("/ibus/list-engines", test_list_engines);
     g_test_add_func ("/ibus/list-active-engines", test_list_active_engines);
     g_test_add_func ("/ibus/create-input-context-async",
                      test_create_input_context_async);
     g_test_add_func ("/ibus/get-engines-by-names", test_get_engines_by_names);
+    g_test_add_func ("/ibus/get-address", test_get_address);
+    g_test_add_func ("/ibus/get-current-input-context",
+                     test_get_current_input_context);
+    g_test_add_func ("/ibus/get-engines", test_get_engines);
+    g_test_add_func ("/ibus/get-global-engine", test_get_global_engine);
+    g_test_add_func ("/ibus/set-preload-engines", test_set_preload_engines);
     g_test_add_func ("/ibus/async-apis", test_async_apis);
     g_test_add_func ("/ibus/bus-new-async", test_bus_new_async);
     g_test_add_func ("/ibus/bus-new-async/list-engines", test_list_engines);
